# pdf_extractor/key_manager.py
import json
from pathlib import Path
import os
from cryptography.fernet import Fernet


class KeyManager:
    def __init__(self):
        self.config_dir = Path.home() / ".pdf_extractor"
        self.keys_file = self.config_dir / "keys.enc"
        self.encryption_key_file = self.config_dir / ".encryption_key"
        self._initialize_storage()

    def _initialize_storage(self):
        """Initialize the storage directory and encryption key"""
        self.config_dir.mkdir(exist_ok=True)

        # Create or load encryption key
        if not self.encryption_key_file.exists():
            encryption_key = Fernet.generate_key()
            self.encryption_key_file.write_bytes(encryption_key)

        self.fernet = Fernet(self.encryption_key_file.read_bytes())

        # Create empty keys file if it doesn't exist
        if not self.keys_file.exists():
            self._save_keys({})

    def _load_keys(self):
        """Load and decrypt keys"""
        if not self.keys_file.exists():
            return {}
        encrypted_data = self.keys_file.read_bytes()
        decrypted_data = self.fernet.decrypt(encrypted_data)
        return json.loads(decrypted_data)

    def _save_keys(self, keys):
        """Encrypt and save keys"""
        encrypted_data = self.fernet.encrypt(json.dumps(keys).encode())
        self.keys_file.write_bytes(encrypted_data)

    def add_key(self, model: str, api_key: str):
        """Add a new API key"""
        keys = self._load_keys()
        keys[model.lower()] = api_key
        self._save_keys(keys)

    def get_key(self, model: str) -> str:
        """Get API key for a model"""
        keys = self._load_keys()
        return keys.get(model.lower())

    def delete_key(self, model: str):
        """Delete API key for a model"""
        keys = self._load_keys()
        if model.lower() in keys:
            del keys[model.lower()]
            self._save_keys(keys)

    def list_keys(self):
        """List all stored models with partial key preview"""
        keys = self._load_keys()
        preview = {}
        for model, key in keys.items():
            # Show only first 4 and last 4 characters
            preview[model] = f"{key[:4]}...{key[-4:]}"
        return preview



# pdf_extractor/extractors/gemini_extractor.py
from pathlib import Path
from google import genai
import PIL.Image
from .base import BaseExtractor


class GeminiExtractor(BaseExtractor):
    def initialize_client(self):
        self.client = genai.Client(api_key=self.api_key)

    def extract_text_from_image(self, image_path: Path) -> str:
        pil_image = PIL.Image.open(image_path)
        response = self.client.models.generate_content(
            model="gemini-2.0-flash",
            contents=[
                "Extract and return ONLY the text from this image. Format it exactly as it appears.",
                pil_image,
            ],
        )
        return response.text



# pdf_extractor/cli.py
import click
from pathlib import Path
from dotenv import load_dotenv
import os
from .extractors import GeminiExtractor, OpenAIExtractor
from .utils import process_pdf
from .key_manager import KeyManager


@click.group()
def cli():
    """PDF Text Extractor CLI"""
    pass


@cli.command()
@click.argument("pdf_path", type=click.Path(exists=True))
@click.option(
    "--model",
    "-m",
    type=click.Choice(["gemini", "openai"]),
    default="gemini",
    help="AI model to use for text extraction",
)
@click.option("--output", "-o", type=click.Path(), help="Output file path")
@click.option("--api-key", help="API key for the chosen model (optional if stored)")
def extract(pdf_path: str, model: str, output: str, api_key: str):
    """Extract text from PDF using AI models."""
    load_dotenv()

    # Try to get API key from different sources
    key_manager = KeyManager()
    if not api_key:
        api_key = key_manager.get_key(model)
        if not api_key:
            api_key = os.getenv(f"{model.upper()}_API_KEY")
            if not api_key:
                raise click.ClickException(
                    f"No API key found for {model}. Please provide it via --api-key option "
                    f"or store it using 'extract-pdf keys add {model} YOUR_KEY'"
                )

    # Select extractor based on model choice
    extractors = {"gemini": GeminiExtractor, "openai": OpenAIExtractor}

    extractor_class = extractors[model]
    extractor = extractor_class(api_key=api_key)
    extractor.initialize_client()

    # Process PDF
    output_path = output or Path(pdf_path).stem + "_extracted.txt"
    text = process_pdf(pdf_path, extractor, output_path)
    click.echo(f"Text extracted and saved to {output_path}")


@cli.group()
def keys():
    """Manage API keys"""
    pass


@keys.command()
@click.argument("model", type=click.Choice(["gemini", "openai"]))
@click.argument("api_key")
def add(model: str, api_key: str):
    """Store an API key for a model"""
    key_manager = KeyManager()
    key_manager.add_key(model, api_key)
    click.echo(f"API key for {model} stored successfully")


@keys.command()
@click.argument("model", type=click.Choice(["gemini", "openai"]))
def delete(model: str):
    """Delete stored API key for a model"""
    key_manager = KeyManager()
    key_manager.delete_key(model)
    click.echo(f"API key for {model} deleted")


@keys.command()
def list():
    """List all stored API keys (partial preview)"""
    key_manager = KeyManager()
    stored_keys = key_manager.list_keys()
    if not stored_keys:
        click.echo("No API keys stored")
        return

    click.echo("Stored API keys:")
    for model, preview in stored_keys.items():
        click.echo(f"{model}: {preview}")


def main():
    cli()


if __name__ == "__main__":
    main()



from setuptools import setup, find_packages

# Read the README file
with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="pdfgenius",
    version="0.1.1",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=find_packages(
        include=["pdfgenius", "pdfgenius.*"]
    ),  # Explicitly include packages
    install_requires=[
        "pdf2image",
        "Pillow",
        "python-dotenv",
        "click",
        "google-generativeai",
        "openai",
        "cryptography",
    ],
    entry_points={
        "console_scripts": [
            "extract-pdf=pdfgenius.cli:main",
        ],
    },
    python_requires=">=3.7",
)



# pdf_extractor/extractors/base.py
from abc import ABC, abstractmethod
from pathlib import Path
from typing import Optional


class BaseExtractor(ABC):
    def __init__(self, api_key: Optional[str] = None):
        self.api_key = api_key

    @abstractmethod
    def extract_text_from_image(self, image_path: Path) -> str:
        """Extract text from a single image"""
        pass

    @abstractmethod
    def initialize_client(self):
        """Initialize the AI client"""
        pass



# pdf_extractor/utils.py
from pathlib import Path
from pdf2image import convert_from_path
from typing import List, Optional, Type
from .extractors.base import BaseExtractor


def process_pdf(
    pdf_path: str, extractor: BaseExtractor, output_path: Optional[str] = None
) -> str:
    # Convert PDF to images
    images = convert_from_path(pdf_path)
    extracted_text = []

    # Create temporary directory for images
    temp_dir = Path("temp_images")
    temp_dir.mkdir(exist_ok=True)

    try:
        for i, image in enumerate(images):
            temp_image_path = temp_dir / f"page_{i}.png"
            image.save(temp_image_path)

            try:
                text = extractor.extract_text_from_image(temp_image_path)
                extracted_text.append(f"--- Page {i+1} ---\n{text}\n")
            except Exception as e:
                print(f"Error processing page {i+1}: {e}")
            finally:
                temp_image_path.unlink(missing_ok=True)

    finally:
        temp_dir.rmdir()

    full_text = "\n".join(extracted_text)

    if output_path:
        with open(output_path, "w", encoding="utf-8") as f:
            f.write(full_text)

    return full_text



# pdf_extractor/extractors/openai_extractor.py
import base64
from pathlib import Path
from openai import OpenAI
from .base import BaseExtractor


class OpenAIExtractor(BaseExtractor):
    def initialize_client(self):
        self.client = OpenAI(api_key=self.api_key)

    def encode_image(self, image_path: Path) -> str:
        with open(image_path, "rb") as image_file:
            return base64.b64encode(image_file.read()).decode("utf-8")

    def extract_text_from_image(self, image_path: Path) -> str:
        base64_image = self.encode_image(image_path)
        response = self.client.chat.completions.create(
            model="gpt-4-vision-preview",
            messages=[
                {
                    "role": "user",
                    "content": [
                        {
                            "type": "text",
                            "text": "Extract and return ONLY the text from this image. Format it exactly as it appears.",
                        },
                        {
                            "type": "image_url",
                            "image_url": {
                                "url": f"data:image/jpeg;base64,{base64_image}"
                            },
                        },
                    ],
                }
            ],
        )
        return response.choices[0].message.content
